@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * netlink/cli/link.h     CLI Link Helpers
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2008-2010 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -15,6 +9,10 @@
 #include <netlink/route/link.h>
 #include <netlink/cli/utils.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern struct rtnl_link *nl_cli_link_alloc(void);
 extern struct nl_cache *nl_cli_link_alloc_cache_family(struct nl_sock *, int);
 extern struct nl_cache *nl_cli_link_alloc_cache_family_flags(struct nl_sock *, int,
@@ -31,4 +29,8 @@ extern void nl_cli_link_parse_txqlen(struct rtnl_link *, char *);
 extern void nl_cli_link_parse_weight(struct rtnl_link *, char *);
 extern void nl_cli_link_parse_ifalias(struct rtnl_link *, char *);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
