@@ -3,12 +3,14 @@
  * Copyright (c) 2012 Cumulus Networks, Inc
  */
 
-#include <string.h>
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
 #include <netlink/object.h>
 #include <netlink/hash.h>
 #include <netlink/hashtable.h>
 
+#include "nl-aux-core/nl-core.h"
+
 /**
  * @ingroup core_types
  * @defgroup hashtable Hashtable
