@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/netfilter/nfnl.c		Netfilter Netlink
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
  * Copyright (c) 2007 Philip Craig <philipc@snapgear.com>
  * Copyright (c) 2007 Secure Computing Corporation
@@ -61,12 +55,17 @@
  * @{
  */
 
-#include <netlink-private/netlink.h>
-#include <netlink/netlink.h>
-#include <netlink/netfilter/nfnl.h>
+#include "nl-default.h"
 
 #include <linux/netfilter/nfnetlink.h>
 
+#include <netlink/netlink.h>
+#include <netlink/netfilter/nfnl.h>
+#include <netlink/data.h>
+#include <netlink/msg.h>
+
+#include "nl-aux-core/nl-core.h"
+
 /**
  * @name Socket Creating
  * @{
