@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/tc.c		Traffic Control
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2011 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -15,14 +9,20 @@
  * @{
  */
 
-#include <netlink-private/netlink.h>
-#include <netlink-private/tc.h>
+#include "nl-default.h"
+
+#include <linux/if_arp.h>
+#include <linux/gen_stats.h>
+
+#include <linux/atm.h>
+
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/route/rtnl.h>
 #include <netlink/route/link.h>
 #include <netlink/route/tc.h>
-#include <netlink-private/route/tc-api.h>
+
+#include "tc-api.h"
 
 /** @cond SKIP */
 
@@ -32,12 +32,13 @@ static struct rtnl_tc_type_ops *tc_type_ops[__RTNL_TC_TYPE_MAX];
 static struct nla_policy tc_policy[TCA_MAX+1] = {
 	[TCA_KIND]	= { .type = NLA_STRING,
 			    .maxlen = TCKINDSIZ },
+	[TCA_CHAIN]	= { .type = NLA_U32 },
 	[TCA_STATS]	= { .minlen = sizeof(struct tc_stats) },
 	[TCA_STATS2]	= { .type = NLA_NESTED },
 };
 
 int tca_parse(struct nlattr **tb, int maxattr, struct rtnl_tc *g,
-	      struct nla_policy *policy)
+	      const struct nla_policy *policy)
 {
 	
 	if (g->ce_mask & TCA_ATTR_OPTS)
@@ -79,6 +80,9 @@ int rtnl_tc_msg_parse(struct nlmsghdr *n, struct rtnl_tc *tc)
 	nla_strlcpy(kind, tb[TCA_KIND], sizeof(kind));
 	rtnl_tc_set_kind(tc, kind);
 
+	if (tb[TCA_CHAIN])
+	        rtnl_tc_set_chain(tc, nla_get_u32(tb[TCA_CHAIN]));
+
 	tm = nlmsg_data(n);
 	tc->tc_family  = tm->tcm_family;
 	tc->tc_ifindex = tm->tcm_ifindex;
@@ -105,11 +109,11 @@ int rtnl_tc_msg_parse(struct nlmsghdr *n, struct rtnl_tc *tc)
 			return err;
 
 		if (tbs[TCA_STATS_BASIC]) {
-			struct gnet_stats_basic *bs;
+			struct gnet_stats_basic bs;
 			
-			bs = nla_data(tbs[TCA_STATS_BASIC]);
-			tc->tc_stats[RTNL_TC_BYTES]	= bs->bytes;
-			tc->tc_stats[RTNL_TC_PACKETS]	= bs->packets;
+			memcpy(&bs, nla_data(tbs[TCA_STATS_BASIC]), sizeof(bs));
+			tc->tc_stats[RTNL_TC_BYTES]	= bs.bytes;
+			tc->tc_stats[RTNL_TC_PACKETS]	= bs.packets;
 		}
 
 		if (tbs[TCA_STATS_RATE_EST]) {
@@ -142,16 +146,17 @@ int rtnl_tc_msg_parse(struct nlmsghdr *n, struct rtnl_tc *tc)
 			goto compat_xstats;
 	} else {
 		if (tb[TCA_STATS]) {
-			struct tc_stats *st = nla_data(tb[TCA_STATS]);
+			struct tc_stats st;
 
-			tc->tc_stats[RTNL_TC_BYTES]	= st->bytes;
-			tc->tc_stats[RTNL_TC_PACKETS]	= st->packets;
-			tc->tc_stats[RTNL_TC_RATE_BPS]	= st->bps;
-			tc->tc_stats[RTNL_TC_RATE_PPS]	= st->pps;
-			tc->tc_stats[RTNL_TC_QLEN]	= st->qlen;
-			tc->tc_stats[RTNL_TC_BACKLOG]	= st->backlog;
-			tc->tc_stats[RTNL_TC_DROPS]	= st->drops;
-			tc->tc_stats[RTNL_TC_OVERLIMITS]= st->overlimits;
+			memcpy(&st, nla_data(tb[TCA_STATS]), sizeof(st));
+			tc->tc_stats[RTNL_TC_BYTES]	= st.bytes;
+			tc->tc_stats[RTNL_TC_PACKETS]	= st.packets;
+			tc->tc_stats[RTNL_TC_RATE_BPS]	= st.bps;
+			tc->tc_stats[RTNL_TC_RATE_PPS]	= st.pps;
+			tc->tc_stats[RTNL_TC_QLEN]	= st.qlen;
+			tc->tc_stats[RTNL_TC_BACKLOG]	= st.backlog;
+			tc->tc_stats[RTNL_TC_DROPS]	= st.drops;
+			tc->tc_stats[RTNL_TC_OVERLIMITS]= st.overlimits;
 
 			tc->ce_mask |= TCA_ATTR_STATS;
 		}
@@ -202,17 +207,22 @@ int rtnl_tc_msg_build(struct rtnl_tc *tc, int type, int flags,
 		.tcm_handle = tc->tc_handle,
 		.tcm_parent = tc->tc_parent,
 	};
-	int err = -NLE_MSGSIZE;
+	int err;
 
 	msg = nlmsg_alloc_simple(type, flags);
 	if (!msg)
 		return -NLE_NOMEM;
 
-	if (nlmsg_append(msg, &tchdr, sizeof(tchdr), NLMSG_ALIGNTO) < 0)
-		goto nla_put_failure;
+	if (nlmsg_append(msg, &tchdr, sizeof(tchdr), NLMSG_ALIGNTO) < 0) {
+		err = -NLE_MSGSIZE;
+		goto out_err;
+	}
 
 	if (tc->ce_mask & TCA_ATTR_KIND)
-	    NLA_PUT_STRING(msg, TCA_KIND, tc->tc_kind);
+		NLA_PUT_STRING(msg, TCA_KIND, tc->tc_kind);
+
+	if (tc->ce_mask & TCA_ATTR_CHAIN)
+	        NLA_PUT_U32(msg, TCA_CHAIN, tc->tc_chain);
 
 	ops = rtnl_tc_get_ops(tc);
 	if (ops && (ops->to_msg_fill || ops->to_msg_fill_raw)) {
@@ -220,21 +230,28 @@ int rtnl_tc_msg_build(struct rtnl_tc *tc, int type, int flags,
 		void *data = rtnl_tc_data(tc);
 
 		if (ops->to_msg_fill) {
-			if (!(opts = nla_nest_start(msg, TCA_OPTIONS)))
-				goto nla_put_failure;
+			if (!(opts = nla_nest_start(msg, TCA_OPTIONS))) {
+				err = -NLE_NOMEM;
+				goto out_err;
+			}
 
 			if ((err = ops->to_msg_fill(tc, data, msg)) < 0)
-				goto nla_put_failure;
+				goto out_err;
 
-			nla_nest_end(msg, opts);
+			if (strcmp("cgroup", tc->tc_kind))
+				nla_nest_end(msg, opts);
+			else
+				nla_nest_end_keep_empty(msg, opts);
 		} else if ((err = ops->to_msg_fill_raw(tc, data, msg)) < 0)
-			goto nla_put_failure;
+			goto out_err;
 	}
 
 	*result = msg;
 	return 0;
 
 nla_put_failure:
+	err = -NLE_NOMEM;
+out_err:
 	nlmsg_free(msg);
 	return err;
 }
@@ -513,7 +530,12 @@ int rtnl_tc_set_kind(struct rtnl_tc *tc, const char *kind)
 	if (tc->ce_mask & TCA_ATTR_KIND)
 		return -NLE_EXIST;
 
-	strncpy(tc->tc_kind, kind, sizeof(tc->tc_kind) - 1);
+	if (   !kind
+	    || strlen (kind) >= sizeof (tc->tc_kind))
+		return -NLE_INVAL;
+
+	_nl_strncpy_assert(tc->tc_kind, kind, sizeof(tc->tc_kind));
+
 	tc->ce_mask |= TCA_ATTR_KIND;
 
 	/* Force allocation of data */
@@ -551,6 +573,34 @@ uint64_t rtnl_tc_get_stat(struct rtnl_tc *tc, enum rtnl_tc_stat id)
 	return tc->tc_stats[id];
 }
 
+/**
+ * Set the chain index of a traffic control object
+ * @arg tc		traffic control object
+ * @arg chain		chain index of traffic control object
+ *
+ */
+void rtnl_tc_set_chain(struct rtnl_tc *tc, uint32_t chain)
+{
+	tc->tc_chain = chain;
+	tc->ce_mask |= TCA_ATTR_CHAIN;
+}
+
+/**
+ * Return chain index of traffic control object
+ * @arg tc		traffic control object
+ * @arg out_value       output argument.
+ *
+ * @return 0 of the output value was successfully returned, or a negative
+ *   error code on failure.
+ */
+int rtnl_tc_get_chain(struct rtnl_tc *tc, uint32_t *out_value)
+{
+	if (!(tc->ce_mask & TCA_ATTR_CHAIN))
+		return -NLE_MISSING_ATTR;
+	*out_value = tc->tc_chain;
+	return 0;
+}
+
 /** @} */
 
 /**
@@ -596,11 +646,7 @@ int rtnl_tc_str2stat(const char *name)
  */
 int rtnl_tc_calc_txtime(int bufsize, int rate)
 {
-	double tx_time_secs;
-	
-	tx_time_secs = (double) bufsize / (double) rate;
-
-	return tx_time_secs * 1000000.;
+	return ((double) bufsize / (double) rate) * 1000000.0;
 }
 
 /**
@@ -619,24 +665,20 @@ int rtnl_tc_calc_txtime(int bufsize, int rate)
  */
 int rtnl_tc_calc_bufsize(int txtime, int rate)
 {
-	double bufsize;
-
-	bufsize = (double) txtime * (double) rate;
-
-	return bufsize / 1000000.;
+	return ((double) txtime * (double) rate) / 1000000.0;
 }
 
 /**
  * Calculate the binary logarithm for a specific cell size
  * @arg cell_size	Size of cell, must be a power of two.
- * @return Binary logirhtm of cell size or a negative error code.
+ * @return Binary logarithm of cell size or a negative error code.
  */
 int rtnl_tc_calc_cell_log(int cell_size)
 {
 	int i;
 
 	for (i = 0; i < 32; i++)
-		if ((1 << i) == cell_size)
+		if ((((uint32_t)1u) << i) == ((uint32_t)cell_size))
 			return i;
 
 	return -NLE_INVAL;
@@ -732,7 +774,7 @@ int rtnl_tc_build_rate_table(struct rtnl_tc *tc, struct rtnl_ratespec *spec,
 
 	for (i = 0; i < RTNL_TC_RTABLE_SIZE; i++) {
 		size = adjust_size((i + 1) << cell_log, spec->rs_mpu, linktype);
-		dst[i] = nl_us2ticks(rtnl_tc_calc_txtime(size, spec->rs_rate));
+		dst[i] = nl_us2ticks(rtnl_tc_calc_txtime64(size, spec->rs_rate64));
 	}
 
 	spec->rs_cell_align = -1;
@@ -771,14 +813,17 @@ int rtnl_tc_clone(struct nl_object *dstobj, struct nl_object *srcobj)
 	struct rtnl_tc *src = TC_CAST(srcobj);
 	struct rtnl_tc_ops *ops;
 
+	dst->tc_opts = NULL;
+	dst->tc_xstats = NULL;
+	dst->tc_subdata = NULL;
+	dst->tc_link = NULL;
+	dst->tc_ops = NULL;
+
 	if (src->tc_link) {
 		nl_object_get(OBJ_CAST(src->tc_link));
 		dst->tc_link = src->tc_link;
 	}
 
-	dst->tc_opts = NULL;
-	dst->tc_xstats = NULL;
-	dst->tc_subdata = NULL;
 	dst->ce_mask &= ~(TCA_ATTR_OPTS |
 	                  TCA_ATTR_XSTATS);
 
@@ -800,18 +845,19 @@ int rtnl_tc_clone(struct nl_object *dstobj, struct nl_object *srcobj)
 		if (!(dst->tc_subdata = nl_data_clone(src->tc_subdata))) {
 			return -NLE_NOMEM;
 		}
-	}
 
-	ops = rtnl_tc_get_ops(src);
-	if (ops && ops->to_clone) {
-		void *a = rtnl_tc_data(dst), *b = rtnl_tc_data(src);
+		/* Warning: if the data contains pointer, then at this point, dst->tc_subdata
+		 * will alias those pointers.
+		 *
+		 * ops->to_clone() MUST fix that.
+		 *
+		 * If the type is actually "struct rtnl_act", then to_clone() must also
+		 * fix dangling "a_next" pointer. */
 
-		if (!a)
-			return 0;
-		else if (!b)
-			return -NLE_NOMEM;
-
-		return ops->to_clone(a, b);
+		ops = rtnl_tc_get_ops(src);
+		if (ops && ops->to_clone) {
+			return ops->to_clone(rtnl_tc_data(dst), rtnl_tc_data(src));
+		}
 	}
 
 	return 0;
@@ -908,22 +954,19 @@ void rtnl_tc_dump_stats(struct nl_object *obj, struct nl_dump_params *p)
 
 	res = nl_cancel_down_bytes(tc->tc_stats[RTNL_TC_BYTES], &unit);
 
-	nl_dump_line(p,
-	             "       %10.2f %3s   %10u   %-10u %-10u %-10u %-10u\n",
-	             res, unit,
-	             tc->tc_stats[RTNL_TC_PACKETS],
-	             tc->tc_stats[RTNL_TC_DROPS],
-	             tc->tc_stats[RTNL_TC_OVERLIMITS],
-	             tc->tc_stats[RTNL_TC_QLEN],
-	             tc->tc_stats[RTNL_TC_BACKLOG]);
+	nl_dump_line(
+		p,
+		"       %10.2f %3s   %10llu   %-10llu %-10llu %-10llu %-10llu\n",
+		res, unit, (long long unsigned)tc->tc_stats[RTNL_TC_PACKETS],
+		(long long unsigned)tc->tc_stats[RTNL_TC_DROPS],
+		(long long unsigned)tc->tc_stats[RTNL_TC_OVERLIMITS],
+		(long long unsigned)tc->tc_stats[RTNL_TC_QLEN],
+		(long long unsigned)tc->tc_stats[RTNL_TC_BACKLOG]);
 
 	res = nl_cancel_down_bytes(tc->tc_stats[RTNL_TC_RATE_BPS], &unit);
 
-	nl_dump_line(p,
-	             "       %10.2f %3s/s %10u/s\n",
-	             res,
-	             unit,
-	             tc->tc_stats[RTNL_TC_RATE_PPS]);
+	nl_dump_line(p, "       %10.2f %3s/s %10llu/s\n", res, unit,
+		     (long long unsigned)tc->tc_stats[RTNL_TC_RATE_PPS]);
 }
 
 uint64_t rtnl_tc_compare(struct nl_object *aobj, struct nl_object *bobj,
@@ -933,14 +976,12 @@ uint64_t rtnl_tc_compare(struct nl_object *aobj, struct nl_object *bobj,
 	struct rtnl_tc *b = TC_CAST(bobj);
 	uint64_t diff = 0;
 
-#define TC_DIFF(ATTR, EXPR) ATTR_DIFF(attrs, TCA_ATTR_##ATTR, a, b, EXPR)
-
-	diff |= TC_DIFF(HANDLE,		a->tc_handle != b->tc_handle);
-	diff |= TC_DIFF(PARENT,		a->tc_parent != b->tc_parent);
-	diff |= TC_DIFF(IFINDEX,	a->tc_ifindex != b->tc_ifindex);
-	diff |= TC_DIFF(KIND,		strcmp(a->tc_kind, b->tc_kind));
-
-#undef TC_DIFF
+#define _DIFF(ATTR, EXPR) ATTR_DIFF(attrs, ATTR, a, b, EXPR)
+	diff |= _DIFF(TCA_ATTR_HANDLE, a->tc_handle != b->tc_handle);
+	diff |= _DIFF(TCA_ATTR_PARENT, a->tc_parent != b->tc_parent);
+	diff |= _DIFF(TCA_ATTR_IFINDEX, a->tc_ifindex != b->tc_ifindex);
+	diff |= _DIFF(TCA_ATTR_KIND, strcmp(a->tc_kind, b->tc_kind));
+#undef _DIFF
 
 	return diff;
 }
@@ -981,7 +1022,7 @@ int rtnl_tc_register(struct rtnl_tc_ops *ops)
 	/*
 	 * Initialiation hack, make sure list is initialized when
 	 * the first tc module registers. Putting this in a
-	 * separate __init would required correct ordering of init
+	 * separate _nl_init would required correct ordering of init
 	 * functions
 	 */
 	if (!init) {
