@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/version.c	Run-time version information
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -24,6 +18,8 @@
  * @{
  */
 
+#include "nl-default.h"
+
 #include <netlink/version.h>
 
 const int      nl_ver_num = LIBNL_VER_NUM;
