@@ -1,17 +1,15 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * nl-list-caches.c	List registered cache types
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2009 Thomas Graf <tgraf@suug.ch>
  */
 
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
 #include <netlink/cli/utils.h>
 
+#include "nl-priv-dynamic-core/object-api.h"
+#include "nl-priv-dynamic-core/cache-api.h"
+
 static void print_usage(void)
 {
 	fprintf(stderr, "Usage: nl-list-caches\n");
