@@ -1,14 +1,10 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * nl-list-sockets.c	Pretty-print /proc/net/netlink
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2009 Thomas Graf <tgraf@suug.ch>
  */
 
+#include "nl-default.h"
+
 #include <netlink/cli/utils.h>
 
 #define PROC_NETLINK "/proc/net/netlink"
