@@ -3,6 +3,14 @@
  * Copyright (c) 2011 Adrian Ban <adrian.ban@mantech.ro>
  */
 
+#include "nl-default.h"
+
+#include <stdio.h>
+
+#include <linux/if_ether.h>
+#include <linux/pkt_cls.h>
+#include <linux/netlink.h>
+
 #include <netlink/route/link.h>
 #include <netlink/route/tc.h>
 #include <netlink/route/qdisc.h>
@@ -11,17 +19,7 @@
 #include <netlink/route/cls/u32.h>
 #include <netlink/route/classifier.h>
 #include <netlink/route/class.h>
-#include <linux/if_ether.h>
-#include <linux/pkt_cls.h>
-
 #include <netlink/attr.h>
-//#include "include/rtnl_u32.h"
-
-#include <stdio.h>
-#include <string.h>
-//#include "include/rtnl_u32_addon.h"
-
-#include <linux/netlink.h>
 
 #define 	TC_HANDLE(maj, min)   (TC_H_MAJ((maj) << 16) | TC_H_MIN(min))
 
